package dsexamples.misc;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import natinst.msl.datasocket.*;

/** DSWriter demonstrates how to use the DataSocket library to write to different
  * data targets.
  */
public class DSWriter extends Applet implements WindowListener {

  Label sStatus, lab;
  TextField sURL;
  DataSocket ds1 = null;
  GraphCanvas grCanvas1 = null;
  Frame fr = null;
  double dValue = 0.1;    // start value

  // Currently, everything is done in init()
  public void init() {
    Choice choice = null;

    setLayout(new BorderLayout());
    Color cLabel = Color.lightGray;
    Color cText = Color.white;

    Panel URLPanel = new Panel();
    URLPanel.setLayout(new BorderLayout());
    URLPanel.add("West", lab = new Label("Target:", Label.LEFT));
    URLPanel.add("Center", sURL = new TextField("dstp://localhost/wave"));
    lab.setBackground(cLabel);
    sURL.setBackground(cText);

    Panel StatusPanel = new Panel();
    StatusPanel.setLayout(new BorderLayout());
    StatusPanel.add("West", lab = new Label("Status:", Label.LEFT));
    StatusPanel.add("Center", sStatus = new Label("Unconnected", Label.LEFT));
    lab.setBackground(cLabel);
    sStatus.setBackground(cLabel);

    Panel ChoicePanel = new Panel();
    ChoicePanel.setLayout(new BorderLayout());
    ChoicePanel.add("Center", choice = new Choice());
    choice.setBackground(cText);

    Panel p1 = new Panel();
    p1.setLayout(new GridLayout(3, 1));
    p1.add("North", URLPanel);
    p1.add("North", ChoicePanel);
    p1.add("North", StatusPanel);
    add("North", p1);

    Panel p = new Panel();
    p.setLayout(new GridLayout(5, 1));
    p.add(new Button("Connect"));
    p.add(new Button("Connect (Manual)"));
    p.add(new Button("Disconnect"));
    p.add(new Button("Update"));
    p.add(new Button("Increment"));
    add("West", p);

    grCanvas1 = new GraphCanvas();
    add("Center", grCanvas1);

    choice.addItem("file:newwave.dsd");
    choice.addItem("dstp://localhost/wave");
    choice.select("dstp://localhost/wave");

    ds1 = new DataSocket();

    // Add listener for DSOnStatusChanged event
    ds1.addDSOnStatusUpdatedListener(new DSOnStatusUpdatedListener() {
      public void DSOnStatusUpdated(DSOnStatusUpdatedEvent e) {
        ds1_DSOnStatusUpdated(e);
      }
    });

  }

  // start with an initial value displayed
  public void start() {
    IncrementValue();
  }

  // Show status
  public void ds1_DSOnStatusUpdated(DSOnStatusUpdatedEvent e) {
    sStatus.setText(ds1.getLastMessage());
  }

  // Respond to button presses, selection of choice
  public boolean action(Event evt, Object arg) {
	  if (evt.target instanceof Choice) {
	    sURL.setText(((Choice)evt.target).getSelectedItem());
   	} else {
	    if ("Connect".equals(arg)) {
		    ds1.connectTo(sURL.getText(), DSAccessModes.cwdsWriteAutoUpdate);
	    } else if ("Disconnect".equals(arg)) {
		    ds1.disconnect();
	    } else if ("Update".equals(arg)) {
        ds1.update();
      } else if ("Connect (Manual)".equals(arg)) {
        ds1.connectTo(sURL.getText(), DSAccessModes.cwdsWrite);
      } else if ("Increment".equals(arg)) {
        IncrementValue();
      }
	  }
	  return true;
  }

  // Increment the waveform value
  private void IncrementValue() {
    dValue = dValue + 0.1;
    if (dValue > 1) dValue = 0.1;

    double[] f = new double[40];
    for (int i=0; i < 40; i++) {
       f[i] = Math.sin(i * dValue);
    }
    grCanvas1.plotY(f);
    try {
       ds1.getData().setValue(f);
    } catch (DSException ex) {
       ex.printStackTrace();
    }
  }

  // close application when window close message is received
  public void windowClosing(WindowEvent e) {
   if (fr != null) {
      fr.setVisible(false);
      fr.dispose();
      System.exit(0);
    }
  }
  public void windowOpened(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}

  public void setFrame(Frame fr1) {
    fr = fr1;
    fr.addWindowListener(this);
  }

  // Create frame and start DSWriter Applet
  public static void main(String args[]) {
	  Frame f = new Frame("DSWriter");

	  DSWriter dsWriter = new DSWriter();
    dsWriter.init();

    Color c = f.getForeground();
    dsWriter.setFrame(f);
    f.setBackground(c);
    f.add("Center", dsWriter);
    f.setSize(400, 200);
	  f.setVisible(true);

	  dsWriter.start();

  }
}

