package dsexamples.misc;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class GraphCanvas extends java.awt.Canvas {

  private Vector m_pData = new Vector(0);
  private Image m_pImage;
  private Graphics m_pGr;
  private boolean m_bPainted = false;
  private Color[] m_cColors = { Color.green, Color.blue, Color.red, Color.yellow};
  private boolean m_bAutoScale = true;
  private int m_iLineWidth = 1;

  private final boolean m_bDrawOffScreen = true;

  // per plot values (if multiple pots are added)
  double m_dYMin = 0, m_dYMax = 10;
  double m_dXMax, m_dXMin;
  int m_iWidth, m_iHeight;

  // eventually, we'll want accessor functions here
  public Color m_cFrame = Color.lightGray;
  public Color m_cBackground = Color.black;
  public Color m_cAxes = Color.black;
  public int m_iFrameWidth = 30;
  public int m_iFrameHeight = 15;

  public GraphCanvas()
  {
    setBackground(m_cBackground);
    enableEvents(AWTEvent.COMPONENT_EVENT_MASK);
  }

  public void setColors(Color[] cColors) { m_cColors = cColors; }
  public Color[] getColors() { return m_cColors; }
  public void setColor(int index, Color c) { m_cColors[index] = c; }
  public void setAutoScale(boolean bAutoScale) { m_bAutoScale = bAutoScale; }
  public boolean getAutoScale() { return m_bAutoScale; }
  public void setYRange(double yMin, double yMax) {
    m_dYMin = yMin;
    m_dYMax = yMax;
    NewDataAvailable();
  }
  public void setLineWidth(int iLineWidth) { m_iLineWidth = iLineWidth; }
  public int getLineWidth() { return m_iLineWidth; }


  public void ClearGraph() {
    ClearImage();
  }

  private void PaintInit() {
    Rectangle rbounds = this.getBounds();

    if (rbounds.width != 0 && rbounds.height != 0) {
      m_iWidth = rbounds.width - m_iFrameWidth;
      m_iHeight = rbounds.height - m_iFrameHeight;

      m_pImage = createImage(rbounds.width, rbounds.height);
      m_pGr = m_pImage.getGraphics();
      drawFrame(m_pGr);
      m_bPainted = true;
    }
  }

  private void NewDataAvailable() {
    calculateRange();
    if (!m_bPainted) {
      PaintInit();
    }
    if (m_bPainted) {
      ClearImage();
      PaintImage();
    }
    repaint();
  }

  public void plotY(double[][] ppd) {

    int nDims = ppd.length;

    m_pData = new Vector(nDims);
    for (int j = 0; j < nDims; j++) {

      int nElts = ppd[j].length;
      Vector arr = new Vector(nElts);
      m_pData.addElement(arr);
      for (int i=0; i < nElts; i++) {
        arr.insertElementAt(new Double(ppd[j][i]), i);
      }
    }

    NewDataAvailable();
  }

  public void plotY(double[] d) {

    m_pData = new Vector(1);
    Vector arr = new Vector(d.length);
    m_pData.addElement(arr);
    for (int i=0; i < d.length; i++) {
      arr.insertElementAt(new Double(d[i]), i);
    }

    NewDataAvailable();
  }

  private double[] getData(int index) {
    double[] d = null;
    int iSize = 0;

    Vector arr = (Vector)(m_pData.elementAt(index));
    d = new double[arr.size()];

    for (int i = 0; i < d.length; i++) {
      d[i] = ((Double)(arr.elementAt(i))).doubleValue();
    }
    return d;
  }

  private void drawFrame(Graphics gr) {
    Rectangle rbounds = this.getBounds();
    int iWidth = rbounds.width;
    int iHeight = rbounds.height;

    if (gr != null) {
      Color c = gr.getColor();
      gr.setColor(m_cFrame);
      gr.fillRect(0, 0, iWidth, iHeight);
      gr.setColor(m_cBackground);
      gr.fillRect(m_iFrameWidth, 0, m_iWidth, m_iHeight);
      gr.setColor(c);

      drawAxes(gr);

    }
  }

  private void drawAxes(Graphics gr) {
    Color c = gr.getColor();
    gr.setColor(m_cAxes);

    FontMetrics fm = gr.getFontMetrics();
    int height = fm.getHeight();

    // y axis
    double dYMax = Math.rint(m_dYMax * 100) / 100;
    double dYMin = Math.rint(m_dYMin * 100) / 100;
    String sYMax, sYMin;
    if (dYMax != m_dYMax) sYMax = Double.toString(dYMax);
      else sYMax = Integer.toString((int)dYMax);
    if (dYMin != m_dYMin) sYMin = Double.toString(dYMin);
      else sYMin = Integer.toString((int)dYMin);

    int iYMaxLen = fm.stringWidth(sYMax);
    int iYMinLen = fm.stringWidth(sYMin);
    gr.drawString(sYMax, m_iFrameWidth - iYMaxLen, height);
    gr.drawString(sYMin, m_iFrameWidth - iYMinLen, m_iHeight);

    // x axis
    double dXMax = Math.rint(m_dXMax * 100) / 100;
    double dXMin = Math.rint(m_dXMin * 100) / 100;
    String sXMax, sXMin;
    if (dXMax != m_dXMax) sXMax = Double.toString(dXMax);
      else sXMax = Integer.toString((int)dXMax);
    if (dXMin != m_dXMin) sXMin = Double.toString(dXMin);
      else sXMin = Integer.toString((int)dXMin);

    int iXMaxLen = fm.stringWidth(sXMax);
    int iXMinLen = fm.stringWidth(sXMax);
    gr.drawString(sXMax, m_iWidth + m_iFrameWidth - iXMaxLen, m_iHeight + m_iFrameHeight);
    gr.drawString(sXMin, m_iFrameWidth, m_iHeight + m_iFrameHeight);

    gr.setColor(c);
  }

  protected void trans(int xin, int yin, Integer xout, Integer yout) {
    xout = new Integer(2 * xin);
    yout = new Integer(yin);
  }

  // should be called only from paint
  protected void calculateRange() {

    double ymin = 0, ymax = 0, xmin = 0, xmax = 0;
    double[] data = null;

    for (int index = 0; index < m_pData.size(); index++) {
      data = getData(index);

      // calculate points
      int points = data.length;
      if (index == 0) { xmax = points; }
        else if (points > xmax) xmax = points;

      if (getAutoScale()) {

        if (index == 0) {
          ymax = data[0];
          ymin = data[0];
        }
        for (int i = 0; i < points; i++) {
          if (data[i] > ymax) ymax = data[i];
          if (data[i] < ymin) ymin = data[i];
        }
      }
    }

    if (m_bAutoScale) {
      m_dYMin = ymin;
      m_dYMax = ymax;
    }

    m_dXMin = xmin;
    m_dXMax = xmax;
  }

  private void ClearImage() {
    m_pGr.clearRect(m_iFrameWidth/2, m_iFrameHeight/2, m_iWidth, m_iHeight);
  }

  // draws graph off-screen
  private void PaintImage() {
    PaintImage(null);
  }

  // draws graph
  private void PaintImage(Graphics gr) {

    double x1, x2, y1, y2, sx1, sx2, sy1, sy2;
    double ymin, ymax, xinc, yinc;

    if (gr == null) {
      gr = m_pGr;
    }
    drawFrame(gr);

    for (int k=0; k < m_pData.size(); k++) {

      double[] data = getData(k);
      int points = data.length;

      if (m_cColors.length > k) {
        gr.setColor(m_cColors[k]);
      }
      else {
        gr.setColor(Color.green);
      }

      if (data != null && points > 0) {

        ymin = m_dYMin;
        ymax = m_dYMax;
        xinc = (double)m_iWidth / (double)(points-1);
        yinc = (double)m_iHeight / (ymax - ymin);

        for (int i = 0; i < points - 1; i++) {
          x1 = i;
          y1 = data[i];
          x2 = i + 1;
          y2 = data[i+1];

          // scale points for plotting to GraphCanvas
          sx1 = x1 * xinc + m_iFrameWidth;
          sy1 = m_iHeight - ((y1 - ymin) * yinc);
          sx2 = x2 * xinc + m_iFrameWidth;
          sy2 = m_iHeight - ((y2 - ymin) * yinc);

          sy1 = sy1 - (int)((m_iLineWidth - 1)/2);
          sy2 = sy2 - (int)((m_iLineWidth - 1)/2);
          for (int lw = 0; lw < m_iLineWidth; lw++) {
            gr.drawLine((int)sx1, (int)sy1 + lw, (int)sx2, (int)sy2 + lw);
          }
        }
      }
    }

  }

  protected void processComponentEvent(ComponentEvent e) {
    /*
    switch (e.getID()) {
      case e.COMPONENT_RESIZED:
        // initialize off-screen image
        PaintInit();
        PaintImage();
        break;
      default: break;
    };*/
    super.processComponentEvent(e);
  }

  // paint is called as a result of
  //   1) resizing (so we need to recalculate the image)
  //   2) data has changed (via plot data call)
  public void paint(Graphics gr) {

    // PaintInit has already been called
    // Just draw off-screen image
    if (m_bDrawOffScreen) {
      gr.drawImage(m_pImage, 0, 0, this);
    }
    else {
      PaintImage(gr);
    }
  }
}
